RSpec.describe NfseCarioca::GerarNfse do
  let(:phone_number) { "2222-2222" }
  let(:zipcode) { "22255777" }
  let(:nota_fiscal_info) do
    {
      identification_number: invoice_number,
      description: "Venda do Serviço XYZ.",
      total: 12.0,
      customer: {
        document_type: "Cnpj",
        document_number: "98924674000187",
        name: "XPTO Tecnologia Ltda.",
        phone_number: phone_number,
        email: "customer@example.com",
        address: {
          street: "Rua qualquer nome",
          number: "33",
          complement: "Sala 1",
          neighborhood: "Seu bairro",
          city_ibge_code: "3550308",
          state_code: "SP",
          zipcode: zipcode
        }
      }
    }
  end

  let!(:invoice_generator) { described_class.new(nota_fiscal_info) }

  describe '#to_xml' do
    let(:invoice_number) { "1024" }
    let(:xml) { invoice_generator.to_xml }

    context "when everything is fine" do
      it 'contains correct identification number' do
        expect(xml).to include %{<Numero>1024</Numero>}
      end

      it 'contains correct description' do
        expect(xml).to include %{<Discriminacao>Venda do Serviço XYZ.}
      end

      it 'contains correct description' do
        expect(xml).to include %{<ValorServicos>12.0</ValorServicos>}
      end

      it 'contains correct provider information' do
        expect(xml).to include %{<Prestador><Cnpj>12390095000186</Cnpj><InscricaoMunicipal>04807995</InscricaoMunicipal></Prestador>}
      end

      it 'contains correct customer information' do
        expect(xml).to include %{<IdentificacaoTomador><CpfCnpj><Cnpj>98924674000187</Cnpj></CpfCnpj></IdentificacaoTomador><RazaoSocial>XPTO Tecnologia Ltda</RazaoSocial><Endereco><Endereco>Rua qualquer nome</Endereco><Numero>33</Numero><Complemento>Sala 1</Complemento><Bairro>Seu bairro</Bairro><CodigoMunicipio>3550308</CodigoMunicipio><Uf>SP</Uf><Cep>22255777</Cep></Endereco>}
      end
    end

    context "when phone is wrong" do
      context "when number is 551148073291" do
        let(:phone_number) { "551148073291" }
        it { expect(xml).to include %{<Telefone>1148073291</Telefone>} }
      end

      context "when number is 313184002584" do
        let(:phone_number) { "313184002584" }
        it { expect(xml).to include %{<Telefone>3184002584</Telefone>} }
      end

      context "when number is 011949860019" do
        let(:phone_number) { "011949860019" }
        it { expect(xml).to include %{<Telefone>11949860019</Telefone>} }
      end

      context "when number is (11) 94986-0019" do
        let(:phone_number) { "(11) 94986-0019" }
        it { expect(xml).to include %{<Telefone>11949860019</Telefone>} }
      end

    end

    context "when zipcode is wrong" do
      context "when number is 05116-050" do
        let(:zipcode) { "05116-050" }
        it { expect(xml).to include %{<Cep>05116050</Cep>} }
      end
    end
  end

  describe "#call" do
    context 'when NfseCarioca rps number does not exist yet', :vcr do
      let(:response) { invoice_generator.execute! }
      let(:invoice_number) { "1042" }

      it 'generates NfseCarioca' do
        expect(response.number).to eq "70"
        expect(response.verification_code).to eq "LLGF-XMCB"
      end
    end

    context 'when NfseCarioca rps number already exist', :vcr do
      let(:response) { invoice_generator.execute! }
      let(:invoice_number) { "1030" }

      it "responds with error" do
        expect(response.error?).to be_truthy
      end

      it "responds with error message E10 " do
        expect(response.errors).to eq [{"Codigo"=>"E10", "Mensagem"=>"RPS já informado.", "Correcao"=>"Para essa Inscrição Municipal/CNPJ já existe um RPS informado com o mesmo número, série e tipo."}]
      end
    end

    context "when NfseCarioca has only the minimum values", :vcr do
      let(:nota_fiscal_info) do
        {
          identification_number: invoice_number,
          description: "Venda do Serviço XYZ.",
          total: 12.0,
          customer: {
            name: "XPTO Tecnologia Ltda.",
            email: "customer@example.com"
          }
        }
      end
      let(:response) { invoice_generator.execute! }
      let(:invoice_number) { "1043" }

      it 'generates NfseCarioca' do
        expect(response.number).to eq "69"
        expect(response.verification_code).to eq "I4RK-NUYD"
      end
    end
  end
end
